pub trait Secret:
Drop
+ Debug
+ Display { }Expand description
A trait that forces an object to implement a zeroizing Drop() as well as Debug and Display that will not log the sensitive contents, even in error or crash-dump scenarios.
pub trait Secret:
Drop
+ Debug
+ Display { }A trait that forces an object to implement a zeroizing Drop() as well as Debug and Display that will not log the sensitive contents, even in error or crash-dump scenarios.